@@ -90,5 +90,48 @@ class TestSuite(unittest.TestCase):
                         with self.assertWarns(UserWarning):
                             imp.reload(glymur.lib.openjp2)
 
+
+@unittest.skipIf(glymur.lib.openjp2.OPENJP2 is None and
+                 glymur.lib.openjpeg.OPENJPEG is None,
+                 "Missing openjp2 library.")
+class TestConfig(unittest.TestCase):
+    """Test suite for reading without proper library in place."""
+
+    def setUp(self):
+        self.jp2file = glymur.data.nemo()
+        self.j2kfile = glymur.data.goodstuff()
+
+    def tearDown(self):
+        pass
+
+    def test_read_without_library(self):
+        """Don't have either openjp2 or openjpeg libraries?  Must error out.
+        """
+        with patch('glymur.lib.openjp2.OPENJP2', new=None):
+            with patch('glymur.lib.openjpeg.OPENJPEG', new=None):
+                with self.assertRaises(glymur.jp2k.LibraryNotFoundError):
+                    glymur.Jp2k(self.jp2file).read()
+
+    def test_read_bands_without_library(self):
+        """Don't have openjp2 library?  Must error out.
+        """
+        with patch('glymur.lib.openjp2.OPENJP2', new=None):
+            with patch('glymur.lib.openjpeg.OPENJPEG', new=None):
+                with self.assertRaises(glymur.jp2k.LibraryNotFoundError):
+                    glymur.Jp2k(self.jp2file).read_bands()
+
+    @unittest.skipIf(os.name == "nt", "NamedTemporaryFile issue on windows")
+    def test_write_without_library(self):
+        """Don't have openjp2 library?  Must error out.
+        """
+        data = glymur.Jp2k(self.j2kfile).read()
+        with patch('glymur.lib.openjp2.OPENJP2', new=None):
+            with patch('glymur.lib.openjpeg.OPENJPEG', new=None):
+                with self.assertRaises(glymur.jp2k.LibraryNotFoundError):
+                    with tempfile.NamedTemporaryFile(suffix='.jp2') as tfile:
+                        ofile = Jp2k(tfile.name, 'wb')
+                        ofile.write(data)
+
+
 if __name__ == "__main__":
     unittest.main()
